"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Lint = require("tslint");
const ts = require("typescript");
/* tslint:disable:max-classes-per-file */
// Based on https://github.com/marcuzy/tslint-origin-ordered-imports-rule
// with a modification to classify paths starting with one of our customRootPaths
// as USER paths, so that they get sorted after node_modules.
// For example, if we had an absolute path of `app-src` aliased to a directory
// within our project, imports from `app-src` or `app-src/*` would throw linter errors
// if they were above node_module imports
class Rule extends Lint.Rules.AbstractRule {
    apply(sourceFile) {
        return this.applyWithWalker(new OriginOrderedImportWalker(sourceFile, this.getOptions()));
    }
}
Rule.FAILURE_STRING = `Import of node_modules must be higher than custom import.`;
Rule.metadata = {
    description: `Strict order of imports (node_modules imports higher than custom imports).`,
    optionExamples: [
        `true`,
        `[true, { "customPathRoots": ["someCustomRoot", "differentCustomRoot"] }]`,
    ],
    options: {
        properties: {
            customPathRoots: {
                type: 'array',
                items: [
                    { type: 'string' },
                ],
                minItems: 1,
            },
        },
    },
    optionsDescription: Lint.Utils.dedent `
            An optional object with \'customPathRoots\' property.
            The property is an array of strings, indicating absolute path
            roots that should be considered custom imports instead of
            node modules.
            `,
    rationale: `Helps maintain a readable style in your codebase.`,
    ruleName: `origin-ordered-imports`,
    type: 'typescript',
    typescriptOnly: false,
};
exports.Rule = Rule;
var SourceType;
(function (SourceType) {
    SourceType[SourceType["USER"] = 0] = "USER";
    SourceType[SourceType["LIB"] = 1] = "LIB";
})(SourceType || (SourceType = {}));
const flowRules = {
    [SourceType.LIB]: [SourceType.USER, SourceType.LIB],
    [SourceType.USER]: [SourceType.USER],
};
class OriginOrderedImportWalker extends Lint.RuleWalker {
    constructor(sourceFile, options) {
        super(sourceFile, options);
        this.nextSourceTypeMayBe = flowRules[SourceType.LIB];
        if (this.getOptions()[0]) {
            this.customPathRoots = this.getOptions()[0].customPathRoots;
        }
    }
    /**
     * For expressions like: import { A, B } from 'foo'
     */
    visitImportDeclaration(node) {
        this.check(node, node.moduleSpecifier.getText());
        super.visitImportDeclaration(node);
    }
    /**
     * For expressions like: import foo = require('foo')
     */
    visitImportEqualsDeclaration(node) {
        let moduleName = ``;
        if (node.moduleReference.kind === ts.SyntaxKind.ExternalModuleReference) {
            const moduleRef = node.moduleReference;
            if (moduleRef.expression && moduleRef.expression.kind === ts.SyntaxKind.StringLiteral) {
                moduleName = moduleRef.expression.text;
            }
        }
        else if (node.moduleReference.kind === ts.SyntaxKind.QualifiedName) {
            moduleName = node.moduleReference.getText();
        }
        this.check(node, moduleName);
        super.visitImportEqualsDeclaration(node);
    }
    check(node, source) {
        const sourceType = this.getSourceType(this.removeQuotes(source));
        if (this.nextSourceTypeMayBe.indexOf(sourceType) === -1) {
            this.addFailureAtNode(node, Rule.FAILURE_STRING);
        }
        else {
            this.nextSourceTypeMayBe = flowRules[sourceType];
        }
    }
    getSourceType(source) {
        const trimmedSource = source.trim();
        let usesCustomPathRoot = false;
        if (this.customPathRoots) {
            this.customPathRoots.find((customPathRoot) => {
                if (trimmedSource === customPathRoot || trimmedSource.startsWith(`${customPathRoot}/`)) {
                    usesCustomPathRoot = true;
                    return true;
                }
                return false;
            });
        }
        return trimmedSource.charAt(0) === `.` || usesCustomPathRoot
            ? SourceType.USER
            : SourceType.LIB;
    }
    removeQuotes(value) {
        if (value && value.length > 1 && (value[0] === `'` || value[0] === `"`)) {
            value = value.substr(1, value.length - 2);
        }
        return value;
    }
}
//# sourceMappingURL=originOrderedImportsRule.js.map