// Shared config between legacy and es6

module.exports = {
    env: {
        es6: true,
    },
    parserOptions: {
        ecmaVersion: 6,
        sourceType: `module`,
    },
    plugins: [
        `import`,
    ],

    settings: {
        'import/resolver': {
            node: {
                extensions: [`.js`, `.jsx`, `.ts`, `.tsx`, `.json`],
            },
        },
    },

    rules: {
        // disallow the following extensions in imports
        'import/extensions': [`error`, `always`, {
            js: `never`,
            jsx: `never`,
            ts: `never`,
            tsx: `never`,
            json: `never`,
        }],

        // enforce import order of node_modules -> absolute paths within project -> relative paths within project
        // for example: 'someNodeModule' -> 'someDir/someFile -> './../someFile'
        // unfortunately, this considers subdirectories of a node module (ex 'redux-saga/effects') to be internal
        "import/order": [`error`, {
            groups: [
                [`builtin`, `external`],
                `internal`,
                [`sibling`, `parent`, `index`],
            ],
        }],

        // Prevent importing things that have not been exported.
        // https://github.com/benmosher/eslint-plugin-import/blob/master/docs/rules/named.md
        'import/named': `error`,

        // When importing `* as something`, verify the members of that import
        // https://github.com/benmosher/eslint-plugin-import/blob/master/docs/rules/namespace.md
        'import/namespace': `error`,

        // Enforce uniform paths. ie: `./../component` => `../component`
        // https://github.com/benmosher/eslint-plugin-import/blob/master/docs/rules/no-useless-path-segments.md
        'import/no-useless-path-segments': `error`,

        // Prevent errors when named imports and defaults could be confused.
        // https://github.com/benmosher/eslint-plugin-import/blob/master/docs/rules/no-named-as-default.md
        // https://github.com/benmosher/eslint-plugin-import/blob/master/docs/rules/no-named-as-default-member.md
        'import/no-named-as-default': `error`,
        'import/no-named-as-default-member': `error`,

        // Use Linux \n line endings
        'linebreak-style': [
            `error`,
            `unix`,
        ],

        // require semicolons (never rely on automatic semicolon insertion)
        'semi': [
            `error`,
            `always`,
        ],

        // require that the last line of an array or an object literal have a comma (for better diffs)
        'comma-dangle': [
            `error`,
            `always-multiline`,
        ],

        // Don't use the __iterator__ variable (non-standard code)
        'no-iterator': [
            `error`,
        ],

        // Force a variable to be const if it is never re-assigned
        'prefer-const': [
            `error`,
        ],

        // Dont modify variables decalerd as const
        'no-const-assign': [
            `error`,
        ],
        // Indent rules!
        'indent': [
            `warn`,
            4, // 4 spaces
            {
                SwitchCase: 1, // but only one space inside a switch statement
            },
        ],

        // Allow changing variables passed as part of the function signature
        'no-param-reassign': [
            `off`,
        ],

        // Turn off max code length rules
        'max-len': [
            `off`,
            {
                code: 150,
            },
        ],

        // Don't require inner functions to have names (allow anonymous)
        'func-names': [
            `off`,
        ],

        // Use stroustrup style braces
        'brace-style': [
            `warn`,
            `stroustrup`,
        ],

        // No spaces between 'function' and its parens e.g. function() not function ()
        'space-before-function-paren': [
            `warn`,
            `never`,
        ],

        // require things which are new'd to be Capitalized
        'new-cap': [
            `error`,
        ],

        // Remove unneeded spaces at the end of a line
        'no-trailing-spaces': [
            `warn`,
        ],

        // Allow chained calls to be on the same line
        'newline-per-chained-call': [
            `off`,
        ],

        // ParseInt shouldn't include the radix.
        'radix': [
            `error`,
            `as-needed`,
        ],

        // How should spaces around semicolons work?
        'semi-spacing': [
            `warn`,
            {
                before: false,
                after: true,
            },
        ],

        // Do not allow console.log, console() messages
        'no-console': [
            `warn`,
        ],
        // Use an explicit test of booleans rather than x ? true : false
        'no-unneeded-ternary': [
            `error`,
            {
                defaultAssignment: true,
            },
        ],

        // Don't allow extra spaces where one will do
        'no-multi-spaces': [
            `warn`,
        ],
        // whitespace rules around .
        'no-whitespace-before-property': [
            `warn`,
        ],
        'padded-blocks': [
            `error`,
        ],

        // Always have a space after a // starting a comment
        'spaced-comment': [
            `warn`,
            `always`,
        ],

        // if (false) {} should trigger an error
        'no-constant-condition': [
            `error`,
        ],

        // Using .call() or .apply() when not needed
        'no-useless-call': [
            `error`,
        ],

        // Creating a setter but not a getter
        'accessor-pairs': [
            `error`,
        ],

        // controls spaces around unary operators
        'space-unary-ops': [
            `warn`,
        ],

        // Warn on TODO and FIXME comments
        'no-warning-comments': [
            `warn`,
        ],

        // Underscores bad.
        'camelcase': [
            `warn`,
        ],

        // spaces around properties in object literals
        'key-spacing': [
            `warn`,
        ],

        // Spacing around commas
        'comma-spacing': [
            `warn`,
        ],

        // Spacing around parens
        'space-in-parens': [
            `warn`,
        ],
        'no-multiple-empty-lines': [
            `warn`,
        ],
        'space-before-blocks': [
            `warn`,
        ],
        'quote-props': [
            `error`,
            `consistent-as-needed`,
        ],
        'no-prototype-builtins': [
            `off`,
        ],
        'no-plusplus': [
            `off`,
        ],

        // Tweak the default airbnb rules to allow ForOfStatement
        'no-restricted-syntax': [
            `error`,
            `ForInStatement`,
            `LabeledStatement`,
            `WithStatement`,
        ],

        // no-return-assign currently broken: https://github.com/eslint/eslint/issues/5150
        'no-return-assign': [
            `off`,
        ],
        'curly': [
            `error`,
            `all`,
        ],
    },
};
