import * as Lint from 'tslint';
import * as ts from 'typescript';

/* tslint:disable:max-classes-per-file */

// Based on https://github.com/marcuzy/tslint-origin-ordered-imports-rule
// with a modification to classify paths starting with one of our customRootPaths
// as USER paths, so that they get sorted after node_modules.
// For example, if we had an absolute path of `app-src` aliased to a directory
// within our project, imports from `app-src` or `app-src/*` would throw linter errors
// if they were above node_module imports

export class Rule extends Lint.Rules.AbstractRule {
    public static FAILURE_STRING = `Import of node_modules must be higher than custom import.`;

    public static metadata: Lint.IRuleMetadata = {
        description: `Strict order of imports (node_modules imports higher than custom imports).`,
        optionExamples: [
            `true`,
            `[true, { "customPathRoots": ["someCustomRoot", "differentCustomRoot"] }]`,
        ],
        options: {
            properties: {
                customPathRoots: {
                    type: 'array',
                    items: [
                        { type: 'string' },
                    ],
                    minItems: 1,
                },
            },
        },
        optionsDescription: Lint.Utils.dedent`
            An optional object with \'customPathRoots\' property.
            The property is an array of strings, indicating absolute path
            roots that should be considered custom imports instead of
            node modules.
            `,
        rationale: `Helps maintain a readable style in your codebase.`,
        ruleName: `origin-ordered-imports`,
        type: 'typescript',
        typescriptOnly: false,
    };

    public apply(sourceFile: ts.SourceFile): Lint.RuleFailure[] {
        return this.applyWithWalker(new OriginOrderedImportWalker(sourceFile, this.getOptions()));
    }
}

enum SourceType {
    USER,
    LIB,
}

const flowRules = {
    [SourceType.LIB]: [SourceType.USER, SourceType.LIB],
    [SourceType.USER]: [SourceType.USER],
};

class OriginOrderedImportWalker extends Lint.RuleWalker {
    private customPathRoots: string[] | undefined;
    constructor(sourceFile: ts.SourceFile, options: Lint.IOptions) {
        super(sourceFile, options);
        if (this.getOptions()[0]) {
            this.customPathRoots = this.getOptions()[0].customPathRoots;
        }
    }
    protected nextSourceTypeMayBe: SourceType[] = flowRules[SourceType.LIB];

    /**
     * For expressions like: import { A, B } from 'foo'
     */
    public visitImportDeclaration(node: ts.ImportDeclaration) {
        this.check(node, node.moduleSpecifier.getText());

        super.visitImportDeclaration(node);
    }

    /**
     * For expressions like: import foo = require('foo')
     */
    public visitImportEqualsDeclaration(node: ts.ImportEqualsDeclaration) {
        let moduleName: string = ``;

        if (node.moduleReference.kind === ts.SyntaxKind.ExternalModuleReference) {
            const moduleRef: ts.ExternalModuleReference = node.moduleReference as ts.ExternalModuleReference;
            if (moduleRef.expression && moduleRef.expression.kind === ts.SyntaxKind.StringLiteral) {
                moduleName = (moduleRef.expression as ts.StringLiteral).text;
            }
        } else if (node.moduleReference.kind === ts.SyntaxKind.QualifiedName) {
            moduleName = node.moduleReference.getText();
        }

        this.check(node, moduleName);

        super.visitImportEqualsDeclaration(node);
    }

    protected check(node: ts.Node, source: string) {
        const sourceType: SourceType = this.getSourceType(
            this.removeQuotes(source),
        );

        if (this.nextSourceTypeMayBe.indexOf(sourceType) === -1) {
            this.addFailureAtNode(node, Rule.FAILURE_STRING);
        } else {
            this.nextSourceTypeMayBe = flowRules[sourceType];
        }
    }

    protected getSourceType(source: string): SourceType {
        const trimmedSource = source.trim();
        let usesCustomPathRoot = false;
        if (this.customPathRoots) {
            this.customPathRoots.find((customPathRoot) => {
                if (trimmedSource === customPathRoot || trimmedSource.startsWith(`${customPathRoot}/`)) {
                    usesCustomPathRoot = true;
                    return true;
                }
                return false;
            });
        }
        return trimmedSource.charAt(0) === `.` || usesCustomPathRoot
            ? SourceType.USER
            : SourceType.LIB;
    }

    protected removeQuotes(value: string): string {
        if (value && value.length > 1 && (value[0] === `'` || value[0] === `"`)) {
            value = value.substr(1, value.length - 2);
        }

        return value;
    }
}
